@@ -23,18 +23,16 @@
 #include "khangman.h"
 
 #include <QApplication>
-#include <QDir>
 #include <QStandardPaths>
 #include <QQuickWidget>
 #include <QQmlContext>
 #include <QQmlEngine>
-#include <QPointer>
 
 #include <KLocalizedContext>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KRandomSequence>
-#include <KNS3/DownloadDialog>
+#include <KRandom>
+#include <KNS3/QtQuickDialogWrapper>
 #include <KHelpMenu>
 
 #include <KEduVocDocument>
@@ -53,7 +51,6 @@ KHangMan::KHangMan()
           m_randomInt(0),
           m_scoreMultiplyingFactor(1),
           m_netScore(0),
-          m_doc(nullptr),
           m_helpMenu(new KHelpMenu(NULL))
 {
     setObjectName(QStringLiteral("KHangMan"));
@@ -77,12 +74,9 @@ KHangMan::KHangMan()
 
     setCentralWidget(m_view);
     scanLanguages();
-    setLevel();
 
     //find the themes
     m_themeFactory.addTheme(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("khangman/themes/standardthemes.xml")));
-
-    loadLanguageSpecialCharacters();
 }
 
 KHangMan::~KHangMan()
@@ -112,7 +106,7 @@ void KHangMan::showHandbook()
 
 KConfigGroup KHangMan::config(const QString &group)
 {
-    return KConfigGroup(KSharedConfig::openConfig(qApp->applicationName() + "rc"), group);
+    return KConfigGroup(KSharedConfig::openConfig(qApp->applicationName() + QStringLiteral("rc")), group);
 }
 
 // ----------------------------------------------------------------
@@ -124,7 +118,7 @@ void KHangMan::setCurrentCategory(int index)
     Prefs::setLevelFile(currentLevel.value());
     Prefs::self()->save();
     m_currentCategory = index;
-    emit currentCategoryChanged();
+    Q_EMIT currentCategoryChanged();
 }
 
 void KHangMan::setCurrentLanguage(int index)
@@ -134,18 +128,16 @@ void KHangMan::setCurrentLanguage(int index)
         m_currentLanguage = index;
         Prefs::self()->save();
         loadLevels();
-        loadLanguageSpecialCharacters();
-        setLevel();
-        emit currentLanguageChanged();
+        Q_EMIT currentLanguageChanged();
     }
 }
 
 void KHangMan::setCurrentTheme(int index)
 {
-    KHMTheme *theme = m_themeFactory.buildTheme(index);
+    const KHMTheme *theme = m_themeFactory.getTheme(index);
     Prefs::setTheme(theme->name());
     Prefs::self()->save();
-    emit currentThemeChanged();
+    Q_EMIT currentThemeChanged();
 }
 
 void KHangMan::readFile()
@@ -159,7 +151,7 @@ void KHangMan::readFile()
             "Please check your installation.",
             Prefs::selectedLanguage(),
             Prefs::levelFile());
-        KMessageBox::sorry (this, mString, i18n("Error"));
+        KMessageBox::error (this, mString, i18n("Error"));
     }
 
     // Detects if file is a kvtml file so that it's a hint enable file
@@ -168,71 +160,60 @@ void KHangMan::readFile()
 
 void KHangMan::nextWord()
 {
-    if (m_randomList.count() == 0) {
-        m_originalWord = m_randomList[0].first;
-        m_hint = m_randomList[0].second;
+    // If there are no words, there's nothing we can do, trying will crash
+    if (m_randomList.isEmpty()) {
         return;
-    } else {
-        m_originalWord = m_randomList[m_randomInt%m_randomList.count()].first;
-        m_originalWord = m_originalWord.toUpper();
-        m_hint = m_randomList[m_randomInt%m_randomList.count()].second;
-
-        emit currentHintChanged();
     }
 
-    if (m_originalWord.isEmpty()) {
-        ++m_randomInt;
-        nextWord();
-    }
+    // Pick words from m_randomList sequentially since it is already shuffled.
+    int whichWord = m_randomInt++ % m_randomList.count();
+    m_originalWord = m_randomList[whichWord].first;
+    m_originalWord = m_originalWord.toUpper();
+    m_hint = m_randomList[whichWord].second;
+
+    Q_EMIT currentHintChanged();
 
     m_currentWord.clear();
 
-    int originalWordSize = m_originalWord.size();
-
-    while (m_currentWord.size() < originalWordSize)
-        m_currentWord.append("_");
-
     // Find dashes, spaces, middot and apostrophe
-    QString search = QStringLiteral("- ·'");
-    Q_FOREACH(const QChar &key, search) {
-        int pos = m_originalWord.indexOf( key );
-        while (pos > 0) {
-            m_currentWord.replace(pos, 1, key);
-            pos = m_originalWord.indexOf( key );
+    const QString search = QStringLiteral("- ·'");
+    for (const QChar &c : m_originalWord) {
+        if (search.contains(c)) {
+            m_currentWord.append(c);
+        } else {
+            m_currentWord.append(QLatin1Char('_'));
         }
     }
 
-    emit currentWordChanged();
-
-    ++m_randomInt;
+    Q_EMIT currentWordChanged();
 }
 
 void KHangMan::slotSetWordsSequence()
 {
-    delete m_doc;
+    //Current level file
+    KEduVocDocument doc(this);
 
-    m_doc = new KEduVocDocument(this);
     ///@todo open returns KEduVocDocument::ErrorCode
-    m_doc->open (QUrl::fromLocalFile (Prefs::levelFile()), KEduVocDocument::FileIgnoreLock);
+    doc.open(QUrl::fromLocalFile(Prefs::levelFile()), KEduVocDocument::FileIgnoreLock);
 
     //how many words in the file
-    int wordCount = m_doc->lesson()->entryCount(KEduVocLesson::Recursive);
+    QList<KEduVocExpression *> words = doc.lesson()->entries(KEduVocLesson::Recursive);
 
     //get the words+hints
-    KRandomSequence randomSequence;
     m_randomList.clear();
-    for (int j = 0; j < wordCount; ++j) {
-        QString hint = m_doc->lesson()->entries (KEduVocLesson::Recursive).at(j)->translation(0)->comment();
-        if (hint.isEmpty() && m_doc->identifierCount() > 0) {
+    for (KEduVocExpression* entry : words) {
+        QString hint = entry->translation(0)->comment();
+        if (hint.isEmpty() && doc.identifierCount() > 0) {
             // if there is no comment or it's empty, use the first translation if there is one
-            hint = m_doc->lesson()->entries(KEduVocLesson::Recursive).at(j)->translation(1)->text();
+            hint = entry->translation(1)->text();
         }
-        if (!m_doc->lesson()->entries(KEduVocLesson::Recursive).at(j)->translation(0)->text().isEmpty()) {
-            m_randomList.append(qMakePair(m_doc->lesson()->entries(KEduVocLesson::Recursive).at(j)->translation(0)->text(), hint));
+        QString text = entry->translation(0)->text();
+        if (!text.isEmpty()) {
+            m_randomList.append(qMakePair(text, hint));
         }
     }
     //shuffle the list
-    randomSequence.randomize(m_randomList);
+    KRandom::shuffle(m_randomList);
 }
 
 QString KHangMan::stripAccents(const QString & original)
@@ -260,7 +241,7 @@ void KHangMan::replaceLetters(const QString& charString)
                 break;
         }
     }
-    emit currentWordChanged();
+    Q_EMIT currentWordChanged();
 }
 
 bool KHangMan::isResolved() const
@@ -271,13 +252,13 @@ bool KHangMan::isResolved() const
 void KHangMan::revealCurrentWord()
 {
     m_currentWord = m_originalWord;
-    emit currentWordChanged();
+    Q_EMIT currentWordChanged();
 }
 
 void KHangMan::calculateNetScore()
 {
     m_netScore = ( m_winCount - m_lossCount ) * m_scoreMultiplyingFactor;
-    emit netScoreChanged();
+    Q_EMIT netScoreChanged();
 }
 
 bool KHangMan::containsChar(const QString &sChar)
@@ -293,7 +274,7 @@ int KHangMan::resolveTime()
 void KHangMan::setResolveTime(int resolveTime)
 {
     Prefs::setResolveTime(resolveTime);
-    emit resolveTimeChanged();
+    Q_EMIT resolveTimeChanged();
 }
 
 bool KHangMan::soundEnabled()
@@ -304,7 +285,7 @@ bool KHangMan::soundEnabled()
 void KHangMan::setSoundEnabled(bool sound)
 {
     Prefs::setSound(sound);
-    emit soundEnabledChanged();
+    Q_EMIT soundEnabledChanged();
 }
 
 QStringList KHangMan::languages()
@@ -321,7 +302,7 @@ void KHangMan::setWinCount(int count)
 {
     m_winCount = count;
     calculateNetScore();
-    emit winCountChanged();
+    Q_EMIT winCountChanged();
 }
 
 int KHangMan::lossCount() const
@@ -333,7 +314,7 @@ void KHangMan::setLossCount(int count)
 {
     m_lossCount = count;
     calculateNetScore();
-    emit lossCountChanged();
+    Q_EMIT lossCountChanged();
 }
 
 int KHangMan::scoreMultiplyingFactor() const
@@ -345,7 +326,7 @@ void KHangMan::setScoreMultiplyingFactor( int factor )
 {
     m_scoreMultiplyingFactor = factor;
     calculateNetScore();
-    emit scoreMultiplyingFactorChanged();
+    Q_EMIT scoreMultiplyingFactorChanged();
 }
 
 int KHangMan::netScore() const
@@ -371,28 +352,22 @@ int KHangMan::currentTheme()
 
 QString KHangMan::backgroundUrl()
 {
-    QStringList themes = m_themeFactory.getNames();
-    int index = themes.indexOf(Prefs::theme());
-    KHMTheme *theme = m_themeFactory.buildTheme(index);
+    const KHMTheme *theme = m_themeFactory.getTheme(currentTheme());
     if (theme) {
-        QString filename = QStandardPaths::locate(QStandardPaths::DataLocation, "themes/" + theme->svgFileName());
-        return filename;
+        return QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("themes/") + theme->svgFileName());
     }
     return QString();
 }
 
 QColor KHangMan::currentThemeLetterColor()
 {
-    // Default to white letters
-    QColor color = "white";
-    QStringList themes = m_themeFactory.getNames();
-    int index = themes.indexOf(Prefs::theme());
-    KHMTheme *theme = m_themeFactory.buildTheme(index);
+    const KHMTheme *theme = m_themeFactory.getTheme(currentTheme());
     if (theme) {
-        color = theme->letterColor();
+        return theme->letterColor();
     }
 
-    return color;
+    // Default to white letters
+    return QColor("white");
 }
 
 QStringList KHangMan::categories()
@@ -409,7 +384,7 @@ QStringList KHangMan::currentWord() const
 {
     QStringList currentWordLetters;
 
-    foreach (const QChar& currentWordLetter, m_currentWord)
+    for (const QChar& currentWordLetter : std::as_const(m_currentWord))
     {
         currentWordLetters.append(currentWordLetter);
     }
@@ -427,46 +402,20 @@ QQmlEngine* KHangMan::getEngine()
     return m_view->engine();
 }
 
-
 QStringList KHangMan::alphabet() const
 {
     QStringList letterList;
-    char c = 'A';
-
-    while( c != 'Z') {
+    for (char c = 'A'; c <= 'Z'; ++c) {
         letterList.append(QChar(c));
-        ++c;
     }
 
-    letterList.append(QChar(c));
     letterList.append(m_specialCharacters);
 
     return letterList;
 }
 
-QStringList KHangMan::languageNames() const
-{
-    QStringList languageCodes = SharedKvtmlFiles::languages();
-    if (languageCodes.isEmpty()) {
-        QApplication::instance()->quit();
-    }
-
-    QStringList languageNames;
-
-    foreach (const QString& languageCode, languageCodes)
-    {
-        QLocale locale(languageCode);
-        QString languageName = locale.nativeLanguageName();
-
-        languageNames.append(languageName);
-    }
-
-    return languageNames;
-}
-
 // ----------------------------------------------------------------
 
-
 void KHangMan::scanLanguages()
 {
     m_languageNames.clear();
@@ -500,94 +449,75 @@ void KHangMan::scanLanguages()
             m_currentLanguage = i;
     }
 
-    emit languagesChanged();
-}
-
-void KHangMan::setLevel()
-{
-    m_currentCategory = Prefs::currentLevel();
-    if (m_currentCategory > m_titleLevels.count()) {
-        m_currentCategory = 0;
-    }
-    emit currentCategoryChanged();
+    Q_EMIT languagesChanged();
 }
 
 void KHangMan::show()
 {
-    if (loadLevels()) { // kvtml files have been found
-        if (m_themeFactory.getQty() > 0) {  // themes present
-            QMainWindow::show();
-            // add the qml view as the main widget
-            QString location = QStandardPaths::locate(QStandardPaths::DataLocation, QStringLiteral("qml/main.qml"));
-            QUrl url = QUrl::fromLocalFile(location);
-            m_view->setSource(url);
-        } else { // themes not present
-            QMessageBox::information(this, i18n("Error"), i18n("No theme files found."));
-            exit(EXIT_FAILURE);
-        }
-    } else { // no kvtml files present
-        QMessageBox::information(this, i18n("Error"), i18n("No kvtml files found."));
+    loadLevels();
+    // kvtml files have been found
+
+    if (m_themeFactory.getQty() == 0) { // themes not present
+        KMessageBox::error(this, i18n("No theme files found."), i18n("Error"));
         exit(EXIT_FAILURE);
     }
+
+    QMainWindow::show();
+    // add the qml view as the main widget
+    QString location = QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("qml/main.qml"));
+    QUrl url = QUrl::fromLocalFile(location);
+    m_view->setSource(url);
 }
 
-bool KHangMan::loadLevels()
+void KHangMan::loadLevels()
 {
     //build the Level combobox menu dynamically depending of the data for each language
     m_titleLevels.clear();
     QStringList levelFilenames = SharedKvtmlFiles::fileNames(Prefs::selectedLanguage());
-    QStringList titles = SharedKvtmlFiles::titles(Prefs::selectedLanguage());
 
-    if (levelFilenames.size() == 0) {
+    if (levelFilenames.isEmpty()) {
         Prefs::setSelectedLanguage(QStringLiteral("en"));
         Prefs::self()->save();
         levelFilenames = SharedKvtmlFiles::fileNames(Prefs::selectedLanguage());
-        titles = SharedKvtmlFiles::titles(Prefs::selectedLanguage());
-    }
-
-    if (levelFilenames.isEmpty()){
-        // no kvtml files found
-        return false;
+        if (levelFilenames.isEmpty()) {
+            KMessageBox::error(this, i18n("No kvtml files found."), i18n("Error"));
+            exit(EXIT_FAILURE);
+        }
     }
 
+    const QStringList titles = SharedKvtmlFiles::titles(Prefs::selectedLanguage());
     Q_ASSERT(levelFilenames.count() == titles.count());
     for(int i = 0; i < levelFilenames.count(); ++i) {
         m_titleLevels.insert(titles.at(i), levelFilenames.at(i));
     }
-    emit categoriesChanged();
+    Q_EMIT categoriesChanged();
 
-    if (!levelFilenames.contains(Prefs::levelFile())) {
-        Prefs::setLevelFile(m_titleLevels.constBegin().value());
-        Prefs::setCurrentLevel(0);
-        m_currentCategory = 0;
-        emit currentCategoryChanged();
-        Prefs::self()->save();
+    int level = Prefs::currentLevel();
+    if (level >= m_titleLevels.count() || !levelFilenames.contains(Prefs::levelFile())) {
+        level = 0;
     }
+    setCurrentCategory(level);
 
-    // don't run off the end of the list
-    if (m_currentCategory!=-1 && m_currentCategory > m_titleLevels.count()) {
-        m_currentCategory = m_titleLevels.count();
-        emit currentCategoryChanged();
-    }
-
-    setLevel();
-
-    return true;
+    loadLanguageSpecialCharacters();
 }
 
 void KHangMan::slotDownloadNewStuff()
 {
-    QPointer<KNS3::DownloadDialog> dialog = new KNS3::DownloadDialog(QStringLiteral("khangman.knsrc"), this);
-    dialog->exec();
-    if (!dialog->changedEntries().isEmpty()) {
-        SharedKvtmlFiles::sortDownloadedFiles();
-        //look for languages dirs installed
-        scanLanguages();
-        //refresh Languages menu
-        setCurrentLanguage(m_languages.indexOf(Prefs::selectedLanguage()));
-    }
+    KNS3::QtQuickDialogWrapper *dialog = new KNS3::QtQuickDialogWrapper(QStringLiteral("khangman.knsrc"), this);
+    dialog->open();
 
-    delete dialog;
+    connect(dialog, &KNS3::QtQuickDialogWrapper::closed, this, [this, dialog] {
+        const QList<KNSCore::EntryInternal> entries = dialog->changedEntries();
+        if ( !entries.isEmpty() ){
+            SharedKvtmlFiles::sortDownloadedFiles();
+            //look for languages dirs installed
+            scanLanguages();
+            //refresh Languages menu
+            setCurrentLanguage(m_languages.indexOf(Prefs::selectedLanguage()));
+        }
+
+        dialog->deleteLater();
+    });
 }
 
 void KHangMan::loadLanguageSpecialCharacters()
@@ -595,40 +525,40 @@ void KHangMan::loadLanguageSpecialCharacters()
     QString lang = Prefs::selectedLanguage();
     if (lang.isEmpty())
         return;
-    bool hasSpecialChars = LangUtils::hasSpecialChars(lang);
 
     m_specialCharacters.clear();
-    if (hasSpecialChars) {
-        QString langFileName=QStringLiteral("khangman/%1.txt").arg(lang);
-        QFile langFile;
+    if (!LangUtils::hasSpecialChars(lang)) {
+        return;
+    }
+
+    QString langFileName=QStringLiteral("khangman/%1.txt").arg(lang);
+    QFile langFile;
+    langFile.setFileName(QStandardPaths::locate(QStandardPaths::GenericDataLocation, langFileName));
+
+    // Let's look in local KDEHOME dir then KNS installs each .txt
+    // in kvtml/<lang> as it installs everything at the same place
+    if (!langFile.exists()) {
+        langFileName = QStringLiteral("apps/kvtml/%1/%1.txt").arg(lang);
         langFile.setFileName(QStandardPaths::locate(QStandardPaths::GenericDataLocation, langFileName));
-
-        // Let's look in local KDEHOME dir then KNS installs each .txt
-        // in kvtml/<lang> as it installs everything at the same place
-        if (!langFile.exists()) {
-            langFileName = QStringLiteral("apps/kvtml/%1/%1.txt").arg(lang);
-            langFile.setFileName(QStandardPaths::locate(QStandardPaths::GenericDataLocation, langFileName));
-        }
-
         if (!langFile.exists()) {
             return;
         }
+    }
 
-        update();
+    update();
 
-        // We open the file and store info into the stream...
-        QFile openFileStream(langFile.fileName());
-        openFileStream.open(QIODevice::ReadOnly);
-        QTextStream readFileStr(&openFileStream);
+    // We open the file and store info into the stream...
+    QFile openFileStream(langFile.fileName());
+    openFileStream.open(QIODevice::ReadOnly);
+    QTextStream readFileStr(&openFileStream);
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        readFileStr.setCodec("UTF-8");
+    readFileStr.setCodec("UTF-8");
 #endif
 
-        // m_specialCharacters contains all the words from the file
-        // FIXME: Better name
-        m_specialCharacters = readFileStr.readAll().split('\n');
-        openFileStream.close();
-    }
+    // m_specialCharacters contains all the words from the file
+    // FIXME: Better name
+    m_specialCharacters = readFileStr.readAll().split(QLatin1Char('\n'));
+    openFileStream.close();
 }
 
 
